/***************************************************************************
                          ezsocket.cpp  -  description                              
                             -------------------                                         
    begin                : Tue Aug 17 1999                                           
    copyright            : (C) 1999 by Joe Nelson                         
    email                : 3nelsons@pressenter.com                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#include "easysocket.h"
#include <iostream>

EasySocket::EasySocket(const EasyAddress& host, unsigned short port) throw(EasyNetError)
: _connected(false), _connectedPort(port)
{
	_connectedTo = new EasyAddress(host);
	Connect(host, port);
}

EasySocket::EasySocket(const int handle)
: _handle(handle), _connected(true)
{
	struct sockaddr_in info;
	unsigned int tempSize = sizeof(struct sockaddr);
	getpeername(handle, (struct sockaddr*)&info, &tempSize);
	_connectedPort = info.sin_port;
	_connectedTo = new EasyAddress(inet_ntoa(info.sin_addr));
}

EasySocket::~EasySocket() throw() {
	if(Connected())
		Disconnect();
	delete _connectedTo;
}

/** Sends data to remote host */
void EasySocket::Write(const string& data) throw(EasyNetError) {
	assert(Connected());

	int bs = 0;
	unsigned int bcount = 0;

	while(bcount < data.size())
	{
		if((bs = write(_handle, data.c_str()+bcount, data.size()-bcount)) == -1)
			throw(EasyNetError(errno, "EasySocket::Write()", "write()"));
		bcount += bs;
	}
}

/** Reads up to numChars into a string over
 * ocket connection. Returns the number of haracters actually read.
 *
 * CONDITION: Must be connected
 * ONDITION: 0 < numChars <= SSIZE_MAX
 */
int EasySocket::Read(string& buffer, int numChars) throw(EasyNetError, std::bad_alloc) {
	assert(Connected());
	assert(0 < numChars <= SSIZE_MAX);

	int retval = 0;
	char *temp = new char[numChars+1];

	if((retval = read(_handle, temp, numChars)) == -1)
		throw(EasyNetError(errno, "EasySocket::Read()", "read()"));
	temp[retval] = '\0';

	buffer = temp;
	delete[] temp;

	return retval;
}

/** If connected, disconnects */
void EasySocket::Disconnect() throw() {
	if(Connected())
		shutdown(_handle, 2);
}

/** Connects to host, at port. If already connected, will disconnect old connection.*/
void EasySocket::Connect(const EasyAddress& host, unsigned short port) throw(EasyNetError) {
	if(Connected())
		Disconnect();

	*_connectedTo = host;
	_connectedPort = port;

	_handle = socket(PF_INET, SOCK_STREAM, 0);
	if(_handle == -1)
		throw(EasyNetError(errno, "EasySocket::Connect()", "socket()"));

	struct sockaddr_in cHost;
	cHost.sin_family      = PF_INET;
	cHost.sin_port        = htons(port);
	cHost.sin_addr.s_addr = inet_addr(host.Addresses()[0].c_str());

	if(connect(_handle, (struct sockaddr*)&cHost, sizeof(struct sockaddr)) == -1)
		throw(EasyNetError(errno, "EasySocket::Connect()", "connect()"));

	_connected = true;
}

/** Returns true if connected, false if not */
bool EasySocket::Connected() const throw() {
	return _connected;
}

/** Returns the host to which this socket is connected.
 *
 * PRECONDITION: must be connected
 */
EasyAddress EasySocket::ConnectedTo() const throw() {
	assert(Connected());
	return *_connectedTo;
}

/** Returns the port to which this socket is connected.
 *
 * PRECONDITION: must be connected
 */
unsigned short EasySocket::ConnectedAtPort() const throw() {
	assert(Connected());
	return _connectedPort;
}
