/***************************************************************************
                          ezsocket.h  -  description                              
                             -------------------                                         
    begin                : Tue Aug 17 1999                                           
    copyright            : (C) 1999 by Joe Nelson                         
    email                : 3nelsons@pressenter.com                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#ifndef EasySOCKET_H
#define EasySOCKET_H

#include <netinet/in.h>
#include <sys/socket.h>
#include <errno.h>
#include <unistd.h>
#include "easyaddress.h"
#include "easyneterror.h"

/**A client TCP/IP stream network socket
  * @author Joe Nelson
  */

class EasySocket {
public:
	EasySocket(const EasyAddress& host, unsigned short port) throw(EasyNetError);
	EasySocket(const int handle);

	/** Returns the port to which this socket is connected.
	 *
	 * PRECONDITION: must be connected
	 */
	unsigned short ConnectedAtPort() const throw();

	/** Returns the host to which this socket is connected.
	 *
	 * PRECONDITION: must be connected
	 */
	EasyAddress ConnectedTo() const throw();

	/** Returns true if connected, false if not */
	bool Connected() const throw();

	/** Connects to host, at port. If already
	 * connected, will disconnect old connection.
	 */
	void Connect(const EasyAddress& host, unsigned short port) throw(EasyNetError);

	/** If connected, disconnects */
	void Disconnect() throw();

	/** Reads up to numChars into a string over
	 * a socket connection. Returns the number of
	 * characters actually read.
	 *
	 * PRECONDITION: must be connected
	 * PRECONDITION: 0 < numChars <= SSIZE_MAX
	 */
	int Read(string& buffer, int numChars) throw(EasyNetError, std::bad_alloc);

	/** Sends data to remote host */
	void Write(const string& data) throw(EasyNetError);

	~EasySocket() throw();

private:
	int _handle;
	bool _connected;
	EasyAddress *_connectedTo;
	int _connectedPort;
};

#endif
